# coding: utf-8
require 'rubygems'
require 'sinatra'
require 'rqrcode'
require 'prawn'
require 'prawn/measurements'
require 'prawn/qrcode'
require 'prawn-svg'
require 'color'
require 'excon'
require 'rmagick'
require 'json'
require 'zlib'

include Prawn::Measurements

# module Prawn
#   module Text
#     module Formatted #:nodoc:
#       # @private
#       class LineWrap #:nodoc:
#         def whitespace()
#           # Wrap by these special characters as well
#           "&:/\\" +
#           "\s\t#{zero_width_space()}"
#         end
#       end
#     end
#   end
# end

module Excon
  class Response
    def json!()
      # Convenience function
      JSON.parse(body)
    end
  end
end

# name -> height, width in millimeters
LABEL_SIZES = {
  "11354" => [32 , 57],
  "99012" => [89 , 36],
  "zebra_large" => [100 , 60],
}

LABEL_SIZE= ENV['LABEL_SIZE'] || "11354"

if !LABEL_SIZES[LABEL_SIZE]
  abort("Bruh moment - not printing, undefined LABEL_SIZE=#{LABEL_SIZE} fails to match any of\n#{LABEL_SIZES}")
end

LPR_CMD_PATH = ENV['LPR_CMD_PATH'] || 'lpr'
LPR_PRINTER_NAME = ENV['LPR_PRINTER_NAME'] || 'DYMO_LabelWriter_450'

def render_identicode(data, id, extent)
  pts = [[0, 0], [0, 1], [1, 1], [1, 0], [0, 0]]

  4.times do |n|
    color = Color::HSL.from_fraction((id % 6) / 6.0, 1.0, 0.3).html[1..6]
    id /= 6

    save_graphics_state do
      soft_mask do
        fill_color 'ffffff'
        polygon = [pts[n], [0.5, 0.5], pts[n+1]].map{ |v| [v[0]*bounds.height, v[1]*bounds.height] }
        fill_polygon(*(polygon))
      end

      print_qr_code data, stroke: false,
                          extent: extent, foreground_color: color,
                          pos: [bounds.left, bounds.top]
    end
  end

  fill_color '000000'
end

DYMO_11354_LABEL_SIZE = [32, 57]
DYMO_LABEL_SIZE = [89, 36]
ZEBRA_LABEL_SIZE = [100, 60]

def render_label(name, qr_text, metadata_text, size: LABEL_SIZES[LABEL_SIZE])
  portrait = size[0] > size[1] # Decide if which orientation prints horizontally on the label
  pdf = Prawn::Document.new(:page_layout => portrait ? :portrait : :landscape, page_size: size.map { |x| mm2pt(x) },
                            margin: [2, 2, 2, 6].map { |x| mm2pt(x) }) do
    base_dir = File.dirname(__FILE__)
    font_families.update("DejaVuSans" => {
      normal: base_dir + "/fonts/DejaVuSans.ttf",
      italic: base_dir + "/fonts/DejaVuSans-Oblique.ttf",
      bold: base_dir + "/fonts/DejaVuSans-Bold.ttf",
      bold_italic: base_dir + "/fonts/DejaVuSans-BoldOblique.ttf"
    })
  
    font 'DejaVuSans'

    # Width of right side
    qr_size = [bounds.height / 2, 27].max

    # Right side
    bounding_box([bounds.right - qr_size, bounds.top], width: qr_size) do
      print_qr_code qr_text, stroke: false,
        foreground_color: '000000',
        extent: bounds.width, margin: 0, pos: bounds.top_left

      text_box metadata_text,
        at: [bounds.right - qr_size, -7], size: 8, align: :right, overflow: :shrink_to_fit
    end

    # Left side
    bounding_box(bounds.top_left, width: bounds.width - qr_size) do
      text_box name,
        size: 40, align: :center, valign: :center, width: bounds.width-10,
        inline_format: true, overflow: :shrink_to_fit, disable_wrap_by_char: true
    end
  end

  pdf.render
end

set :bind, '0.0.0.0'

before do
  request.body.rewind
  @json_payload = JSON.parse request.body.read

  # Validate and make available from all handlers
  @name = @json_payload["name"] || halt(400, "Request JSON needs a 'name' field")
  @qr_text = @json_payload["qr_text"] || halt(400, "Request JSON needs a 'qr_text' field")
  @meta_text = @json_payload["meta_text"] || ""
end

post '/api/1/render/pdf' do
  
  headers["Content-Type"] = "application/pdf; charset=utf8"
  render_label(@name, @qr_text, @meta_text)
end

post '/api/1/render/png' do
  headers["Content-Type"] = "image/png"
  img = Magick::ImageList.new()
  img = img.from_blob(render_label(@name, @qr_text, @meta_text)){ self.density = 200 }.first
  img.format = 'png'
  img.background_color = 'white'
  img.to_blob
end

post '/api/1/print' do
  temp = Tempfile.new('labelmaker')
  temp.write(render_label(@name, @qr_text, @meta_text))
  temp.close
  cmd = "#{LPR_CMD_PATH} -P #{LPR_PRINTER_NAME} '#{temp.path}'"
  system(cmd) || puts("Bruh moment (not printing) - running `#{cmd}` failed")

end
